package com.github.gkutiel.filter.mime;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;
import static java.text.MessageFormat.format;
import static javax.xml.parsers.DocumentBuilderFactory.newInstance;

import java.io.StringWriter;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.github.gkutiel.filter.Textual;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomWriter;

public abstract class Xml<T> extends Textual {

	@Inherited
	@Target(TYPE)
	@Retention(RUNTIME)
	public @interface Xsl {
		String value();
	}

	private static final XStream X_STREAM = new XStream();

	public static String toString(final Document doc) throws Exception {
		final StringWriter stringWriter = new StringWriter();
		TransformerFactory.newInstance().newTransformer().transform(new DOMSource(doc), new StreamResult(stringWriter));
		return stringWriter.toString();
	}

	public Xml() {
		super("application/xml");
	}

	private void addXslIfPresent(final Document doc) {
		final Xsl xsl = getClass().getAnnotation(Xsl.class);
		if (xsl == null) return;
		final String stylesheet = format("type=''text/xsl'' href=''{0}''", xsl.value());
		doc.insertBefore(doc.createProcessingInstruction("xml-stylesheet", stylesheet), doc.getDocumentElement());
	}

	@Override
	protected String get() {
		try {
			final Document doc = newInstance().newDocumentBuilder().newDocument();
			X_STREAM.marshal(xml(), new DomWriter(doc));
			addXslIfPresent(doc);
			return toString(doc);
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	protected abstract T xml();
}
